import sys
import argparse
from pathlib import Path
import numpy as np
import ast
from imface.utils import deepface_util as utils

def main():
    parser = argparse.ArgumentParser("imface cli for image vector")
    parser.add_argument("--represent", help="represent image embed vector", default=False)
    parser.add_argument("-v", "--version", help="version", default=False, action="store_true")
    subparsers = parser.add_subparsers(title="subcommands", dest="command")
    distance_command = subparsers.add_parser("distance", help="to get distance between two vector")
    distance_parser = subparsers.add_parser("distance", help="Calculate cosine distance between two vectors.")
    distance_parser.add_argument("-s", "--source", required=True, help="Source vector (space-separated)")
    distance_parser.add_argument("-t", "--target", required=True, help="Target vector (space-separated)")
    parser.add_argument("--extract", help="extract face, and only allowed to extrace one face, inset the image path", default=False)
    parser.add_argument("--treshold", help="get treshold", default=False, action="store_true")
    args = parser.parse_args()

    if args.version:
        print("0.0.0.1.1")
        exit(0)
    elif args.treshold:
        treshold = utils.getTreshold()
        print(treshold)
        exit(0)

    elif args.represent:
        target_image = Path(args.represent)

        if not target_image.exists():
            print("image not exist")
            raise SystemExit(1)
        try:
            embed = utils.getEmbeddingVector(str(target_image))
            print(embed)
        except Exception as e:
            print("error " + repr(e))

    elif args.extract:
        target_image = Path(args.extract)

        if not target_image.exists():
            print("image not exist")
            raise SystemExit(1)
        try:
            data = utils.extractFace(str(target_image))
            if len(data) > 1:
                print("error only allowed one face")
            else:
                print(data[0]['embedding'])
        except Exception as e:
            print("error" + repr(e))

    elif args.command == "distance":
        if args.source and args.target:
            # Convert the string representation of vectors to lists of floats
            source_vector = ast.literal_eval(args.source)
            target_vector = ast.literal_eval(args.target)

            # Convert the lists to numpy arrays with the correct data type
            source_vector = np.array(source_vector, dtype=np.float32)
            target_vector = np.array(target_vector, dtype=np.float32)
            res = utils.getDistance(source_vector, target_vector)
            print(res)

    if __name__ == "__main__":
        main()


import numpy as np


def findCosineDistance(source_representation, test_representation):
    a = np.matmul(np.transpose(source_representation), test_representation)
    b = np.sum(np.multiply(source_representation, source_representation))
    c = np.sum(np.multiply(test_representation, test_representation))
    return 1 - (a / (np.sqrt(b) * np.sqrt(c)))


def findEuclideanDistance(source_representation, test_representation):
    if isinstance(source_representation, list):
        source_representation = np.array(source_representation)

    if isinstance(test_representation, list):
        test_representation = np.array(test_representation)

    euclidean_distance = source_representation - test_representation
    euclidean_distance = np.sum(np.multiply(euclidean_distance, euclidean_distance))
    euclidean_distance = np.sqrt(euclidean_distance)
    return euclidean_distance


def l2_normalize(x):
    return x / np.sqrt(np.sum(np.multiply(x, x)))


def findThreshold(model_name, distance_metric):

    base_threshold = {"cosine": 0.40, "euclidean": 0.55, "euclidean_l2": 0.75}

    thresholds = {
        "VGG-Face": {"cosine": 0.40, "euclidean": 0.60, "euclidean_l2": 0.86},
        "Facenet": {"cosine": 0.40, "euclidean": 10, "euclidean_l2": 0.80},
        "Facenet512": {"cosine": 0.30, "euclidean": 23.56, "euclidean_l2": 1.04},
        "ArcFace": {"cosine": 0.68, "euclidean": 4.15, "euclidean_l2": 1.13},
        "Dlib": {"cosine": 0.07, "euclidean": 0.6, "euclidean_l2": 0.4},
        "SFace": {"cosine": 0.593, "euclidean": 10.734, "euclidean_l2": 1.055},
        "OpenFace": {"cosine": 0.10, "euclidean": 0.55, "euclidean_l2": 0.55},
        "DeepFace": {"cosine": 0.23, "euclidean": 64, "euclidean_l2": 0.64},
        "DeepID": {"cosine": 0.015, "euclidean": 45, "euclidean_l2": 0.17},
    }

    threshold = thresholds.get(model_name, base_threshold).get(distance_metric, 0.4)

    return threshold


from setuptools import setup, find_packages
from setuptools.command.install import install
from pathlib import Path
import gdown
import os

home_dir = Path.home()
def downloadWeights():
    facenet512_url = "https://github.com/serengil/deepface_models/releases/download/v1.0/facenet512_weights.h5"
    retinaface_url = "https://github.com/serengil/deepface_models/releases/download/v1.0/retinaface.h5"
    if os.path.isfile(str(home_dir) + "/.deepface/weights/facenet512_weights.h5") != True:
        print("facenet512_weights.h5 will be downloaded...")

        output = str(home_dir) + "/.deepface/weights/facenet512_weights.h5"
        gdown.download(facenet512_url, output, quiet=False)
    if os.path.isfile(str(home_dir) + "/.deepface/weights/retinaface.h5") != True:
        print("retinaface.h5 will be downloaded...")

        output = str(home_dir) + "/.deepface/weights/retinaface.h5"
        gdown.download(retinaface_url, output, quiet=False)

class CustomInstallCommand(install):
    def run(self):
        install.run(self)
        downloadWeights()

with open("README.md", "r") as file:
    description = file.read()

requirements = ["deepface"]

setup(
    name='imface',
    version='0.0.0.1.1',
    install_requires=requirements,
    packages=find_packages(),
    include_package_data=True,
    entry_points={"console_scripts": ["imface=imface.main:main"]},
    author="Achmad Alfazari",
    license="License :: OSI Approved :: MIT License",
    classifiers=[
        "Programming Language :: Python",
    ],
    long_description=description,
    long_description_content_type="text/markdown",
    python_requires=">=3.5.5",
    cmdclass={
        'install': CustomInstallCommand,
    },
)


from deepface import DeepFace
from deepface.commons import functions, realtime, distance as dst


models = [
  "VGG-Face", 
  "Facenet", 
  "Facenet512", 
  "OpenFace", 
  "DeepFace", 
  "DeepID", 
  "ArcFace", 
  "Dlib", 
  "SFace",
]

backends = [
    "retinaface",
    "opencv"
]

def getEmbeddingVector(path: str):
    embed = []
    data = DeepFace.represent(path, model_name=models[2], enforce_detection=False, detector_backend=backends[1])
    for imgdata in data:
        embed.append(imgdata['embedding'])
    
    return embed

def extractFace(path: str):
    data = DeepFace.represent(path, model_name=models[2], enforce_detection=True, detector_backend=backends[1])
    return data

def getDistance(source, target):
    dist = dst.findCosineDistance(source, target)
    return dist

def getTreshold():
    threshold = dst.findThreshold(models[2], "cosine")
    return threshold